// Generated by CoffeeScript 1.12.7
(function() {
  var Opcode, Reader, Script, classification, ecc, fingerprint, fingerprints, i, len, ref;

  ecc = require('ecc-tools');

  Reader = require('./util/reader');

  Opcode = require('./opcode');

  Script = (function() {
    function Script(_raw) {
      this._raw = _raw != null ? _raw : '';
      if (!(this instanceof Script)) {
        return new Script(this._raw);
      }
      if (this._raw instanceof Script) {
        return this._raw;
      }
      this._pos = 0;
      this.stack = [];
      this.opcodes = [];
      this._reader = Reader(this._raw);
      while (!this._reader.eof()) {
        this.opcodes.push(Opcode(this._reader));
      }
    }

    Script.prototype.asm = function() {
      var identifiers, op;
      identifiers = (function() {
        var i, len, ref, results;
        ref = this.opcodes;
        results = [];
        for (i = 0, len = ref.length; i < len; i++) {
          op = ref[i];
          results.push(op.identifier);
        }
        return results;
      }).call(this);
      return identifiers.join(' ');
    };

    Script.prototype.buf = function() {
      var op;
      return Buffer.concat((function() {
        var i, len, ref, results;
        ref = this.opcodes;
        results = [];
        for (i = 0, len = ref.length; i < len; i++) {
          op = ref[i];
          results.push(op.buf);
        }
        return results;
      }).call(this));
    };

    Script.prototype.hex = function() {
      return this.buf().toString('hex');
    };

    Script.prototype.codes = function() {
      var op;
      return Buffer.concat((function() {
        var i, len, ref, results;
        ref = this.opcodes;
        results = [];
        for (i = 0, len = ref.length; i < len; i++) {
          op = ref[i];
          results.push(op.code);
        }
        return results;
      }).call(this));
    };

    Script.prototype.classify = function() {
      var ref;
      return (ref = Script.fingerprints[this.fingerprint()]) != null ? ref : 'non-standard';
    };

    Script.prototype.meta = function() {
      var c;
      switch (c = this.classify()) {
        case 'p2pkhm':
          return [c, this.opcodes[5].data];
        default:
          return [c];
      }
    };

    Script.prototype.fingerprint = function() {
      var identifiers, op;
      identifiers = (function() {
        var i, len, ref, results;
        ref = this.opcodes;
        results = [];
        for (i = 0, len = ref.length; i < len; i++) {
          op = ref[i];
          results.push(op.group || op.identifier);
        }
        return results;
      }).call(this);
      return identifiers.join(' ');
    };

    Script.prototype.destination = function() {
      switch (this.classify()) {
        case 'p2pkh':
        case 'p2pkhm':
          return [this.opcodes[2].data.toString('hex')];
        default:
          return [];
      }
    };

    return Script;

  })();

  Script.classifications = {
    p2pkh: ['OP_DUP OP_HASH160 <data> OP_EQUALVERIFY OP_CHECKSIG'],
    p2pkhm: ['OP_DUP OP_HASH160 <data> OP_EQUALVERIFY OP_CHECKSIG <data> OP_DROP']
  };

  Script.fingerprints = {};

  ref = Script.classifications;
  for (classification in ref) {
    fingerprints = ref[classification];
    for (i = 0, len = fingerprints.length; i < len; i++) {
      fingerprint = fingerprints[i];
      Script.fingerprints[fingerprint] = classification;
    }
  }

  Script.p2pkh = function(hash) {
    if (Buffer.isBuffer(hash)) {
      hash = hash.toString('hex');
    }
    return Script('76a914' + hash + '88ac');
  };

  Script.p2pkh_sig = function(private_key, sighash, type) {
    var public_key, signature;
    if (type == null) {
      type = new Buffer('01', 'hex');
    }
    sighash = ecc.reverse(sighash);
    public_key = ecc.publicKey(private_key, true);
    signature = Buffer.concat([ecc.sign(sighash, private_key), type]);
    return Script(signature.length.toString(16) + signature.toString('hex') + '21' + public_key.toString('hex'));
  };

  module.exports = Script;

}).call(this);
