chai = require 'chai'
expect = chai.expect

ecc = require 'ecc-tools'

Output = require '../src/output'

p2pkhm_spkq = '76a91459288e0ec97813bab904867cdcae0d681f6cce5188ac1c73706b71ca325ab67a8f1decb61546a8950da859a08601000000000075'
p2pkhm_spkg = '76a9146872715a6aeaab180807322b8dc6c0bef7d4419f88ac0c73706b6700e1f5050000000075'

hash = ecc.sha256ripemd160('hello world')

describe 'Output', ->
  describe '#instance', ->
    it 'should return an Output instance', ->
      output = Output()
      expect(output).to.be.instanceof(Output)

  describe 'meta', ->
    it 'should return assets for a p2pkhm with spkq metadata', ->
      output = Output(amount: 10, script: p2pkhm_spkq)
      expect(output.meta()).to.eql({'59a80d95a84615b6ec1d8f7ab65a32ca': 100000})

    it 'should return assets for a p2pkhm with spkg metadata', ->
      output = Output(amount: 10, script: p2pkhm_spkg)
      expect(output.meta()).to.eql('100000000')

  describe 'buf', ->
    it 'should return the buffer for the output', ->
      output = Output(amount: 10, script: p2pkhm_spkq)
      expect(output.buf().toString('hex')).to.eql('0a0000000000000037' + p2pkhm_spkq)

  describe 'p2pkh', ->
    it 'should create a output with p2pkh script', ->
      output = Output.p2pkh(hash, 10)
      expect(output.buf().toString('hex')).to.eql('0a000000000000001976a914' + hash.toString('hex') + '88ac')


