chai = require 'chai'
expect = chai.expect


Parser = require '../../src/util/parser'

class Note
  constructor: (@raw) ->
    if !(@ instanceof Note) then return new Note(@raw)

Note.template = [
  ['version', 'uint8']
  ['message', 'vardata']
]


describe 'Parser', ->
  describe '#instance', ->
    it 'should return a Parser instance', ->
      parser = Parser()
      expect(parser).to.be.instanceof(Parser)

  describe 'parse', ->
    it 'should accept a buffer and parse it using the template', ->
      parser = Parser([['version', 'uint8'], ['msg','vardata']])
      result = parser.parse('0308deadbeef')
      expect(result.version).to.equal(3)
      expect(result.msg.toString('hex')).to.equal('deadbeef')

    it 'should accept recursively handle arrays in a template', ->
      parser = Parser([['version', 'uint8'], ['msg','uint8'], ['notes', [['body', 'uint8']]]])
      result = parser.parse('0304020506')
      expect(result).to.eql({
        version: 3
        msg: 4
        notes: [{body: 5}, {body: 6}]
      })

    it 'should create instances for constructors with a template attribute', ->
      parser = Parser(Note)
      result = parser.parse('0308deadbeef')
      expect(result).to.be.an.instanceof(Note)
      expect(result.raw.message.toString('hex')).to.equal('deadbeef')

    it 'should create a single instance', ->
      parser = Parser([['note', Note]])
      result = parser.parse('010308deadbeef')
      expect(result.note).to.be.an.instanceof(Note)

    it 'should create an array of instances', ->
      parser = Parser([['notes', [Note]]])
      result = parser.parse('010308deadbeef')
      expect(result.notes).to.be.an.instanceof(Array)
      expect(result.notes[0]).to.be.an.instanceof(Note)
